/****************************************************************************
** Meta object code from reading C++ file 'address.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kcontacts-6.26.0/src/address.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'address.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9KContacts7AddressE_t {};
} // unnamed namespace

template <> constexpr inline auto KContacts::Address::qt_create_metaobjectdata<qt_meta_tag_ZN9KContacts7AddressE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KContacts::Address",
        "formatted",
        "",
        "KContacts::AddressFormatStyle",
        "style",
        "realName",
        "orgaName",
        "id",
        "isEmpty",
        "type",
        "Type",
        "typeLabel",
        "postOfficeBox",
        "extended",
        "street",
        "locality",
        "region",
        "postalCode",
        "country",
        "label",
        "geo",
        "KContacts::Geo",
        "formattedAddress",
        "geoUri",
        "QUrl",
        "TypeFlag",
        "Dom",
        "Intl",
        "Postal",
        "Parcel",
        "Home",
        "Work",
        "Pref"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'formatted'
        QtMocHelpers::MethodData<QString(KContacts::AddressFormatStyle, const QString &, const QString &) const>(1, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
        }}),
        // Method 'formatted'
        QtMocHelpers::MethodData<QString(KContacts::AddressFormatStyle, const QString &) const>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QString, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 },
        }}),
        // Method 'formatted'
        QtMocHelpers::MethodData<QString(KContacts::AddressFormatStyle) const>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QString, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'id'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'isEmpty'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'type'
        QtMocHelpers::PropertyData<Type>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'typeLabel'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'postOfficeBox'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'extended'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'street'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'locality'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'region'
        QtMocHelpers::PropertyData<QString>(16, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'postalCode'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'country'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'label'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'geo'
        QtMocHelpers::PropertyData<KContacts::Geo>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'formattedAddress'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'geoUri'
        QtMocHelpers::PropertyData<QUrl>(23, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'Type'
        QtMocHelpers::EnumData<Type>(10, 25, QMC::EnumIsFlag).add({
            {   26, TypeFlag::Dom },
            {   27, TypeFlag::Intl },
            {   28, TypeFlag::Postal },
            {   29, TypeFlag::Parcel },
            {   30, TypeFlag::Home },
            {   31, TypeFlag::Work },
            {   32, TypeFlag::Pref },
        }),
    };
    return QtMocHelpers::metaObjectData<Address, qt_meta_tag_ZN9KContacts7AddressE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN9KContacts7AddressE[] = {
    QMetaObject::SuperData::link<KContacts::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KContacts::Address::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KContacts7AddressE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KContacts7AddressE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN9KContacts7AddressE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9KContacts7AddressE_t>.metaTypes,
    nullptr
} };

void KContacts::Address::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Address *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QString _r = _t->formatted((*reinterpret_cast<std::add_pointer_t<KContacts::AddressFormatStyle>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 1: { QString _r = _t->formatted((*reinterpret_cast<std::add_pointer_t<KContacts::AddressFormatStyle>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { QString _r = _t->formatted((*reinterpret_cast<std::add_pointer_t<KContacts::AddressFormatStyle>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->id(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isEmpty(); break;
        case 2: QtMocHelpers::assignFlags<Type>(_v, _t->type()); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->typeLabel(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->postOfficeBox(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->extended(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->street(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->locality(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->region(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->postalCode(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->country(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->label(); break;
        case 12: *reinterpret_cast<KContacts::Geo*>(_v) = _t->geo(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->formattedPostalAddress(); break;
        case 14: *reinterpret_cast<QUrl*>(_v) = _t->geoUri(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setId(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setType(*reinterpret_cast<Type*>(_v)); break;
        case 4: _t->setPostOfficeBox(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setExtended(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setStreet(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setLocality(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setRegion(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setPostalCode(*reinterpret_cast<QString*>(_v)); break;
        case 10: _t->setCountry(*reinterpret_cast<QString*>(_v)); break;
        case 11: _t->setLabel(*reinterpret_cast<QString*>(_v)); break;
        case 12: _t->setGeo(*reinterpret_cast<KContacts::Geo*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
