/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<types.h>)
#  include <types.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_org_kde_contacts()
{
    qmlRegisterModule("org.kde.contacts", 254, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<AddressForeign>("org.kde.contacts", 254);
    qmlRegisterEnum<KContacts::Address::Type>("KContacts::Address::Type");
    qmlRegisterEnum<KContacts::Address::TypeFlag>("KContacts::Address::TypeFlag");
    qmlRegisterTypesAndRevisions<AddresseeForeign>("org.kde.contacts", 254);
    qmlRegisterTypesAndRevisions<EmailForeign>("org.kde.contacts", 254);
    qmlRegisterEnum<KContacts::Email::Type>("KContacts::Email::Type");
    qmlRegisterEnum<KContacts::Email::TypeFlag>("KContacts::Email::TypeFlag");
    qmlRegisterTypesAndRevisions<GeoForeign>("org.kde.contacts", 254);
    qmlRegisterTypesAndRevisions<ImppForeign>("org.kde.contacts", 254);
    {
        Q_CONSTINIT static auto metaType = QQmlPrivate::metaTypeForNamespace(
            [](const QtPrivate::QMetaTypeInterface *) {return &KContacts::staticMetaObject;},
            "KContacts");
        QMetaType(&metaType).id();
    }
    qmlRegisterNamespaceAndRevisions(&KContacts::staticMetaObject, "org.kde.contacts", 254, nullptr, &KContactForeign::staticMetaObject, nullptr);
    qmlRegisterEnum<KContacts::AddressFormatStyle>("KContacts::AddressFormatStyle");
    qmlRegisterEnum<KContacts::AddressFormatField>("KContacts::AddressFormatField");
    qmlRegisterEnum<KContacts::AddressFormatFields>("KContacts::AddressFormatFields");
    qmlRegisterEnum<KContacts::AddressFormatField>("KContacts::AddressFormatField");
    qmlRegisterEnum<KContacts::AddressFormatScriptPreference>("KContacts::AddressFormatScriptPreference");
    qmlRegisterEnum<KContacts::AddressFormatPreference>("KContacts::AddressFormatPreference");
    qmlRegisterTypesAndRevisions<PhoneNumberForeign>("org.kde.contacts", 254);
    qmlRegisterEnum<KContacts::PhoneNumber::Type>("KContacts::PhoneNumber::Type");
    qmlRegisterEnum<KContacts::PhoneNumber::TypeFlag>("KContacts::PhoneNumber::TypeFlag");
    qmlRegisterTypesAndRevisions<PictureForeign>("org.kde.contacts", 254);
    QT_WARNING_POP
    qmlRegisterModule("org.kde.contacts", 254, 254);
}

static const QQmlModuleRegistration orgkdecontactsRegistration("org.kde.contacts", qml_register_types_org_kde_contacts);
