/****************************************************************************
** Meta object code from reading C++ file 'phonenumber.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kcontacts-6.26.0/src/phonenumber.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'phonenumber.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9KContacts11PhoneNumberE_t {};
} // unnamed namespace

template <> constexpr inline auto KContacts::PhoneNumber::qt_create_metaobjectdata<qt_meta_tag_ZN9KContacts11PhoneNumberE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KContacts::PhoneNumber",
        "id",
        "number",
        "normalizedNumber",
        "type",
        "Type",
        "typeLabel",
        "isEmpty",
        "isPreferred",
        "supportsSms",
        "TypeFlag",
        "Home",
        "Work",
        "Msg",
        "Pref",
        "Voice",
        "Fax",
        "Cell",
        "Video",
        "Bbs",
        "Modem",
        "Car",
        "Isdn",
        "Pcs",
        "Pager",
        "Undefined"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'id'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'number'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'normalizedNumber'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'type'
        QtMocHelpers::PropertyData<Type>(4, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'typeLabel'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'isEmpty'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isPreferred'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'supportsSms'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'Type'
        QtMocHelpers::EnumData<Type>(5, 10, QMC::EnumIsFlag).add({
            {   11, TypeFlag::Home },
            {   12, TypeFlag::Work },
            {   13, TypeFlag::Msg },
            {   14, TypeFlag::Pref },
            {   15, TypeFlag::Voice },
            {   16, TypeFlag::Fax },
            {   17, TypeFlag::Cell },
            {   18, TypeFlag::Video },
            {   19, TypeFlag::Bbs },
            {   20, TypeFlag::Modem },
            {   21, TypeFlag::Car },
            {   22, TypeFlag::Isdn },
            {   23, TypeFlag::Pcs },
            {   24, TypeFlag::Pager },
            {   25, TypeFlag::Undefined },
        }),
    };
    return QtMocHelpers::metaObjectData<PhoneNumber, qt_meta_tag_ZN9KContacts11PhoneNumberE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KContacts::PhoneNumber::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KContacts11PhoneNumberE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KContacts11PhoneNumberE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9KContacts11PhoneNumberE_t>.metaTypes,
    nullptr
} };

void KContacts::PhoneNumber::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<PhoneNumber *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->id(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->number(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->normalizedNumber(); break;
        case 3: QtMocHelpers::assignFlags<Type>(_v, _t->type()); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->typeLabel(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isEmpty(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isPreferred(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->supportsSms(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setId(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setNumber(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setType(*reinterpret_cast<Type*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
