/****************************************************************************
** Meta object code from reading C++ file 'addressee.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kcontacts-6.26.0/src/addressee.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'addressee.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9KContacts9AddresseeE_t {};
} // unnamed namespace

template <> constexpr inline auto KContacts::Addressee::qt_create_metaobjectdata<qt_meta_tag_ZN9KContacts9AddresseeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KContacts::Addressee",
        "isEmpty",
        "uid",
        "name",
        "formattedName",
        "familyName",
        "givenName",
        "additionalName",
        "prefix",
        "suffix",
        "nickName",
        "birthday",
        "birthdayHasTime",
        "mailer",
        "geo",
        "KContacts::Geo",
        "title",
        "role",
        "organization",
        "department",
        "note",
        "productId",
        "revision",
        "sortString",
        "url",
        "KContacts::ResourceLocatorUrl",
        "extraUrls",
        "QList<KContacts::ResourceLocatorUrl>",
        "realName",
        "assembledName",
        "preferredEmail",
        "emails",
        "QList<KContacts::Email>",
        "phoneNumbers",
        "QList<KContacts::PhoneNumber>",
        "addresses",
        "QList<KContacts::Address>",
        "categories",
        "customs",
        "changed",
        "impps",
        "QList<KContacts::Impp>",
        "anniversary",
        "assistantsName",
        "blogFeed",
        "QUrl",
        "managersName",
        "office",
        "profession",
        "spousesName",
        "photo",
        "KContacts::Picture",
        "deathDate"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'isEmpty'
        QtMocHelpers::PropertyData<bool>(1, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'uid'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'formattedName'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'familyName'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'givenName'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'additionalName'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'prefix'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'suffix'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'nickName'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'birthday'
        QtMocHelpers::PropertyData<QDateTime>(11, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'birthdayHasTime'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'mailer'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'geo'
        QtMocHelpers::PropertyData<KContacts::Geo>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(16, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'role'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'organization'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'department'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'note'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'productId'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'revision'
        QtMocHelpers::PropertyData<QDateTime>(22, QMetaType::QDateTime, QMC::DefaultPropertyFlags),
        // property 'sortString'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'url'
        QtMocHelpers::PropertyData<KContacts::ResourceLocatorUrl>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'extraUrls'
        QtMocHelpers::PropertyData<QList<KContacts::ResourceLocatorUrl>>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'realName'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'assembledName'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'preferredEmail'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'emails'
        QtMocHelpers::PropertyData<QList<KContacts::Email>>(31, 0x80000000 | 32, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'phoneNumbers'
        QtMocHelpers::PropertyData<QList<KContacts::PhoneNumber>>(33, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'addresses'
        QtMocHelpers::PropertyData<QList<KContacts::Address>>(35, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'categories'
        QtMocHelpers::PropertyData<QStringList>(37, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'customs'
        QtMocHelpers::PropertyData<QStringList>(38, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'changed'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'impps'
        QtMocHelpers::PropertyData<QList<KContacts::Impp>>(40, 0x80000000 | 41, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'anniversary'
        QtMocHelpers::PropertyData<QDate>(42, QMetaType::QDate, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'assistantsName'
        QtMocHelpers::PropertyData<QString>(43, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'blogFeed'
        QtMocHelpers::PropertyData<QUrl>(44, 0x80000000 | 45, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'managersName'
        QtMocHelpers::PropertyData<QString>(46, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'office'
        QtMocHelpers::PropertyData<QString>(47, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'profession'
        QtMocHelpers::PropertyData<QString>(48, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'spousesName'
        QtMocHelpers::PropertyData<QString>(49, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'photo'
        QtMocHelpers::PropertyData<KContacts::Picture>(50, 0x80000000 | 51, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'deathDate'
        QtMocHelpers::PropertyData<QDateTime>(52, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Addressee, qt_meta_tag_ZN9KContacts9AddresseeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN9KContacts9AddresseeE[] = {
    QMetaObject::SuperData::link<KContacts::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KContacts::Addressee::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KContacts9AddresseeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KContacts9AddresseeE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN9KContacts9AddresseeE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9KContacts9AddresseeE_t>.metaTypes,
    nullptr
} };

void KContacts::Addressee::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Addressee *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isEmpty(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->uid(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->formattedName(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->familyName(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->givenName(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->additionalName(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->prefix(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->suffix(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->nickName(); break;
        case 10: *reinterpret_cast<QDateTime*>(_v) = _t->birthday(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->birthdayHasTime(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->mailer(); break;
        case 13: *reinterpret_cast<KContacts::Geo*>(_v) = _t->geo(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        case 15: *reinterpret_cast<QString*>(_v) = _t->role(); break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->organization(); break;
        case 17: *reinterpret_cast<QString*>(_v) = _t->department(); break;
        case 18: *reinterpret_cast<QString*>(_v) = _t->note(); break;
        case 19: *reinterpret_cast<QString*>(_v) = _t->productId(); break;
        case 20: *reinterpret_cast<QDateTime*>(_v) = _t->revision(); break;
        case 21: *reinterpret_cast<QString*>(_v) = _t->sortString(); break;
        case 22: *reinterpret_cast<KContacts::ResourceLocatorUrl*>(_v) = _t->url(); break;
        case 23: *reinterpret_cast<QList<KContacts::ResourceLocatorUrl>*>(_v) = _t->extraUrlList(); break;
        case 24: *reinterpret_cast<QString*>(_v) = _t->realName(); break;
        case 25: *reinterpret_cast<QString*>(_v) = _t->assembledName(); break;
        case 26: *reinterpret_cast<QString*>(_v) = _t->preferredEmail(); break;
        case 27: *reinterpret_cast<QList<KContacts::Email>*>(_v) = _t->emailList(); break;
        case 28: *reinterpret_cast<QList<KContacts::PhoneNumber>*>(_v) = _t->phoneNumbers(); break;
        case 29: *reinterpret_cast<QList<KContacts::Address>*>(_v) = _t->addresses(); break;
        case 30: *reinterpret_cast<QStringList*>(_v) = _t->categories(); break;
        case 31: *reinterpret_cast<QStringList*>(_v) = _t->customs(); break;
        case 32: *reinterpret_cast<bool*>(_v) = _t->changed(); break;
        case 33: *reinterpret_cast<QList<KContacts::Impp>*>(_v) = _t->imppList(); break;
        case 34: *reinterpret_cast<QDate*>(_v) = _t->anniversary(); break;
        case 35: *reinterpret_cast<QString*>(_v) = _t->assistantsName(); break;
        case 36: *reinterpret_cast<QUrl*>(_v) = _t->blogFeed(); break;
        case 37: *reinterpret_cast<QString*>(_v) = _t->managersName(); break;
        case 38: *reinterpret_cast<QString*>(_v) = _t->office(); break;
        case 39: *reinterpret_cast<QString*>(_v) = _t->profession(); break;
        case 40: *reinterpret_cast<QString*>(_v) = _t->spousesName(); break;
        case 41: *reinterpret_cast<KContacts::Picture*>(_v) = _t->photo(); break;
        case 42: *reinterpret_cast<QDateTime*>(_v) = _t->deathDate(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setUid(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setFormattedName(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setFamilyName(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setGivenName(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setAdditionalName(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setPrefix(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setSuffix(*reinterpret_cast<QString*>(_v)); break;
        case 10: _t->setBirthdayProperty(*reinterpret_cast<QDateTime*>(_v)); break;
        case 12: _t->setMailer(*reinterpret_cast<QString*>(_v)); break;
        case 13: _t->setGeo(*reinterpret_cast<KContacts::Geo*>(_v)); break;
        case 17: _t->setDepartment(*reinterpret_cast<QString*>(_v)); break;
        case 18: _t->setNote(*reinterpret_cast<QString*>(_v)); break;
        case 19: _t->setProductId(*reinterpret_cast<QString*>(_v)); break;
        case 21: _t->setSortString(*reinterpret_cast<QString*>(_v)); break;
        case 22: _t->setUrl(*reinterpret_cast<KContacts::ResourceLocatorUrl*>(_v)); break;
        case 23: _t->setExtraUrlList(*reinterpret_cast<QList<KContacts::ResourceLocatorUrl>*>(_v)); break;
        case 27: _t->setEmailList(*reinterpret_cast<QList<KContacts::Email>*>(_v)); break;
        case 28: _t->setPhoneNumbers(*reinterpret_cast<QList<KContacts::PhoneNumber>*>(_v)); break;
        case 29: _t->setAddresses(*reinterpret_cast<QList<KContacts::Address>*>(_v)); break;
        case 30: _t->setCategories(*reinterpret_cast<QStringList*>(_v)); break;
        case 32: _t->setChanged(*reinterpret_cast<bool*>(_v)); break;
        case 33: _t->setImppList(*reinterpret_cast<QList<KContacts::Impp>*>(_v)); break;
        case 34: _t->setAnniversary(*reinterpret_cast<QDate*>(_v)); break;
        case 35: _t->setAssistantsName(*reinterpret_cast<QString*>(_v)); break;
        case 36: _t->setBlogFeed(*reinterpret_cast<QUrl*>(_v)); break;
        case 37: _t->setManagersName(*reinterpret_cast<QString*>(_v)); break;
        case 38: _t->setOffice(*reinterpret_cast<QString*>(_v)); break;
        case 39: _t->setProfession(*reinterpret_cast<QString*>(_v)); break;
        case 40: _t->setSpousesName(*reinterpret_cast<QString*>(_v)); break;
        case 41: _t->setPhoto(*reinterpret_cast<KContacts::Picture*>(_v)); break;
        case 42: _t->setDeathDate(*reinterpret_cast<QDateTime*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
