/****************************************************************************
** Meta object code from reading C++ file 'types.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kcontacts-6.26.0/src/qml/types.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'types.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14AddressForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto AddressForeign::qt_create_metaobjectdata<qt_meta_tag_ZN14AddressForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AddressForeign",
        "QML.Foreign",
        "KContacts::Address",
        "QML.Element",
        "address",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<AddressForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject AddressForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14AddressForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14AddressForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14AddressForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN16AddresseeForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto AddresseeForeign::qt_create_metaobjectdata<qt_meta_tag_ZN16AddresseeForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AddresseeForeign",
        "QML.Foreign",
        "KContacts::Addressee",
        "QML.Element",
        "addressee",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<AddresseeForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject AddresseeForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16AddresseeForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16AddresseeForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16AddresseeForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN12EmailForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto EmailForeign::qt_create_metaobjectdata<qt_meta_tag_ZN12EmailForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "EmailForeign",
        "QML.Foreign",
        "KContacts::Email",
        "QML.Element",
        "email",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "construct"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<EmailForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject EmailForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12EmailForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12EmailForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12EmailForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN18PhoneNumberForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto PhoneNumberForeign::qt_create_metaobjectdata<qt_meta_tag_ZN18PhoneNumberForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PhoneNumberForeign",
        "QML.Foreign",
        "KContacts::PhoneNumber",
        "QML.Element",
        "phoneNumber"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<PhoneNumberForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject PhoneNumberForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18PhoneNumberForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18PhoneNumberForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18PhoneNumberForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN11ImppForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto ImppForeign::qt_create_metaobjectdata<qt_meta_tag_ZN11ImppForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ImppForeign",
        "QML.Foreign",
        "KContacts::Impp",
        "QML.Element",
        "impp"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<ImppForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject ImppForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ImppForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ImppForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11ImppForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN14PictureForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto PictureForeign::qt_create_metaobjectdata<qt_meta_tag_ZN14PictureForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PictureForeign",
        "QML.Foreign",
        "KContacts::Picture",
        "QML.Element",
        "picture"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<PictureForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject PictureForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PictureForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PictureForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14PictureForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10GeoForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto GeoForeign::qt_create_metaobjectdata<qt_meta_tag_ZN10GeoForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GeoForeign",
        "QML.Foreign",
        "KContacts::Geo",
        "QML.Element",
        "geo"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<GeoForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject GeoForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10GeoForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10GeoForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10GeoForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN15KContactForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto KContactForeign::qt_create_metaobjectdata<qt_meta_tag_ZN15KContactForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KContactForeign",
        "QML.Element",
        "KContacts",
        "QML.Foreign",
        "QML.ForeignIsNamespace",
        "true"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    2 },
            {    4,    5 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN15KContactForeignE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN15KContactForeignE =
    KContactForeign::qt_create_metaobjectdata<qt_meta_tag_ZN15KContactForeignE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN15KContactForeignE =
    qt_staticMetaObjectContent_ZN15KContactForeignE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN15KContactForeignE =
    qt_staticMetaObjectContent_ZN15KContactForeignE.relocatingData;

Q_CONSTINIT const QMetaObject KContactForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN15KContactForeignE.stringdata,
    qt_staticMetaObjectStaticContent_ZN15KContactForeignE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN15KContactForeignE.metaTypes,
    nullptr
} };

QT_WARNING_POP
