/****************************************************************************
** Meta object code from reading C++ file 'namespace.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kcontacts-6.26.0/src/namespace.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'namespace.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9KContactsE_t {};
} // unnamed namespace

template <> constexpr inline auto KContacts::qt_create_metaobjectdata<qt_meta_tag_ZN9KContactsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KContacts",
        "AddressFormatStyle",
        "Postal",
        "MultiLineDomestic",
        "MultiLineInternational",
        "SingleLineDomestic",
        "SingleLineInternational",
        "GeoUriQuery",
        "AddressFormatField",
        "NoField",
        "Country",
        "Region",
        "Locality",
        "DependentLocality",
        "SortingCode",
        "PostalCode",
        "StreetAddress",
        "Organization",
        "Name",
        "PostOfficeBox",
        "AddressFormatFields",
        "AddressFormatScriptPreference",
        "Local",
        "Latin",
        "AddressFormatPreference",
        "Generic",
        "Business"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AddressFormatStyle'
        QtMocHelpers::EnumData<AddressFormatStyle>(1, 1, QMC::EnumIsScoped).add({
            {    2, AddressFormatStyle::Postal },
            {    3, AddressFormatStyle::MultiLineDomestic },
            {    4, AddressFormatStyle::MultiLineInternational },
            {    5, AddressFormatStyle::SingleLineDomestic },
            {    6, AddressFormatStyle::SingleLineInternational },
            {    7, AddressFormatStyle::GeoUriQuery },
        }),
        // enum 'AddressFormatField'
        QtMocHelpers::EnumData<AddressFormatField>(8, 8, QMC::EnumIsScoped).add({
            {    9, AddressFormatField::NoField },
            {   10, AddressFormatField::Country },
            {   11, AddressFormatField::Region },
            {   12, AddressFormatField::Locality },
            {   13, AddressFormatField::DependentLocality },
            {   14, AddressFormatField::SortingCode },
            {   15, AddressFormatField::PostalCode },
            {   16, AddressFormatField::StreetAddress },
            {   17, AddressFormatField::Organization },
            {   18, AddressFormatField::Name },
            {   19, AddressFormatField::PostOfficeBox },
        }),
        // flag 'AddressFormatFields'
        QtMocHelpers::EnumData<AddressFormatFields>(20, 8, QMC::EnumIsFlag | QMC::EnumIsScoped).add({
            {    9, AddressFormatField::NoField },
            {   10, AddressFormatField::Country },
            {   11, AddressFormatField::Region },
            {   12, AddressFormatField::Locality },
            {   13, AddressFormatField::DependentLocality },
            {   14, AddressFormatField::SortingCode },
            {   15, AddressFormatField::PostalCode },
            {   16, AddressFormatField::StreetAddress },
            {   17, AddressFormatField::Organization },
            {   18, AddressFormatField::Name },
            {   19, AddressFormatField::PostOfficeBox },
        }),
        // enum 'AddressFormatScriptPreference'
        QtMocHelpers::EnumData<AddressFormatScriptPreference>(21, 21, QMC::EnumIsScoped).add({
            {   22, AddressFormatScriptPreference::Local },
            {   23, AddressFormatScriptPreference::Latin },
        }),
        // enum 'AddressFormatPreference'
        QtMocHelpers::EnumData<AddressFormatPreference>(24, 24, QMC::EnumIsScoped).add({
            {   25, AddressFormatPreference::Generic },
            {   26, AddressFormatPreference::Business },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN9KContactsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN9KContactsE =
    KContacts::qt_create_metaobjectdata<qt_meta_tag_ZN9KContactsE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN9KContactsE =
    qt_staticMetaObjectContent_ZN9KContactsE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN9KContactsE =
    qt_staticMetaObjectContent_ZN9KContactsE.relocatingData;

Q_CONSTINIT const QMetaObject KContacts::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN9KContactsE.stringdata,
    qt_staticMetaObjectStaticContent_ZN9KContactsE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN9KContactsE.metaTypes,
    nullptr
} };

QT_WARNING_POP
