/****************************************************************************
** Meta object code from reading C++ file 'resourcelocatorurl.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kcontacts-6.26.0/src/resourcelocatorurl.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'resourcelocatorurl.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9KContacts18ResourceLocatorUrlE_t {};
} // unnamed namespace

template <> constexpr inline auto KContacts::ResourceLocatorUrl::qt_create_metaobjectdata<qt_meta_tag_ZN9KContacts18ResourceLocatorUrlE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KContacts::ResourceLocatorUrl",
        "url",
        "QUrl",
        "isValid",
        "type",
        "Type",
        "isPreferred",
        "resourceLabel",
        "TypeFlag",
        "Unknown",
        "Home",
        "Work",
        "Profile",
        "Ftp",
        "Reservation",
        "AppInstallPage",
        "Other"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'url'
        QtMocHelpers::PropertyData<QUrl>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'isValid'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'type'
        QtMocHelpers::PropertyData<Type>(4, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'isPreferred'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'resourceLabel'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'Type'
        QtMocHelpers::EnumData<Type>(5, 8, QMC::EnumIsFlag).add({
            {    9, TypeFlag::Unknown },
            {   10, TypeFlag::Home },
            {   11, TypeFlag::Work },
            {   12, TypeFlag::Profile },
            {   13, TypeFlag::Ftp },
            {   14, TypeFlag::Reservation },
            {   15, TypeFlag::AppInstallPage },
            {   16, TypeFlag::Other },
        }),
    };
    return QtMocHelpers::metaObjectData<ResourceLocatorUrl, qt_meta_tag_ZN9KContacts18ResourceLocatorUrlE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KContacts::ResourceLocatorUrl::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KContacts18ResourceLocatorUrlE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KContacts18ResourceLocatorUrlE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9KContacts18ResourceLocatorUrlE_t>.metaTypes,
    nullptr
} };

void KContacts::ResourceLocatorUrl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<ResourceLocatorUrl *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->url(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isValid(); break;
        case 2: QtMocHelpers::assignFlags<Type>(_v, _t->type()); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isPreferred(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->resourceLabel(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setUrl(*reinterpret_cast<QUrl*>(_v)); break;
        case 2: _t->setType(*reinterpret_cast<Type*>(_v)); break;
        case 3: _t->setPreferred(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
