/****************************************************************************
** Meta object code from reading C++ file 'email.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kcontacts-6.26.0/src/email.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'email.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9KContacts5EmailE_t {};
} // unnamed namespace

template <> constexpr inline auto KContacts::Email::qt_create_metaobjectdata<qt_meta_tag_ZN9KContacts5EmailE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KContacts::Email",
        "Email",
        "",
        "mail",
        "email",
        "isValid",
        "type",
        "Type",
        "isPreferred",
        "TypeFlag",
        "Unknown",
        "Home",
        "Work",
        "Other"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'email'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'isValid'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'type'
        QtMocHelpers::PropertyData<Type>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'isPreferred'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'Type'
        QtMocHelpers::EnumData<Type>(7, 9, QMC::EnumIsFlag).add({
            {   10, TypeFlag::Unknown },
            {   11, TypeFlag::Home },
            {   12, TypeFlag::Work },
            {   13, TypeFlag::Other },
        }),
    };
    using Constructor = QtMocHelpers::NoType;
    QtMocHelpers::UintData qt_constructors {
        QtMocHelpers::ConstructorData<Constructor(const QString &)>(2, QMC::AccessPublic, {{
            { QMetaType::QString, 3 },
        }}),
    };
    return QtMocHelpers::metaObjectData<Email, qt_meta_tag_ZN9KContacts5EmailE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors);
}
Q_CONSTINIT const QMetaObject KContacts::Email::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KContacts5EmailE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KContacts5EmailE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9KContacts5EmailE_t>.metaTypes,
    nullptr
} };

void KContacts::Email::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Email *>(_o);
    if (_c == QMetaObject::CreateInstance) {
        switch (_id) {
        case 0: { Email *_r = new Email((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<void**>(_a[0]) = _r; } break;
        default: break;
        }
    }
    if (_c == QMetaObject::ConstructInPlace) {
        switch (_id) {
        case 0: { new (_a[0]) Email((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); } break;
        default: break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->mail(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isValid(); break;
        case 2: QtMocHelpers::assignFlags<Type>(_v, _t->type()); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isPreferred(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setEmail(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setType(*reinterpret_cast<Type*>(_v)); break;
        case 3: _t->setPreferred(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
